import { Injectable, Injector, Inject } from "@angular/core";
import { Router } from "@angular/router";
import {
  HttpInterceptor,
  HttpRequest,
  HttpHandler,
  HttpErrorResponse,
  HttpSentEvent,
  HttpHeaderResponse,
  HttpProgressEvent,
  HttpResponse,
  HttpUserEvent
} from "@angular/common/http";
import { Observable, of, throwError } from "rxjs";
import { mergeMap, catchError } from "rxjs/operators";
import { environment } from "src/environments/environment";

/**
 * 默认HTTP拦截器，其注册细节见 `app.module.ts`
 */
@Injectable()
export class DefaultInterceptor implements HttpInterceptor {
  constructor(private injector: Injector) {}

  // get msg(): NzMessageService {
  //   return this.injector.get(NzMessageService);
  // }

  private goTo(url: string) {
    setTimeout(() => this.injector.get(Router).navigateByUrl(url));
  }

  private handleData(
    event: HttpResponse<any> | HttpErrorResponse
  ): Observable<any> {
    // 可能会因为 `throw` 导出无法执行 `_HttpClient` 的 `end()` 操作
    // this.injector.get(_HttpClient).end();
    // 业务处理：一些通用操作
    switch (event.status) {
      case 200:
      case 201:
        //  业务层级错误处理，以下是假定restful有一套统一输出格式（指不管成功与否都有相应的数据格式）情况下进行处理
        //  例如响应内容：
        //  错误内容：{ code: 1, msg: '非法参数' }
        //  正确内容：{ code: 0, data: {  },msg:'合法参数'}
        //  则以下代码片断可直接适用
        if (event instanceof HttpResponse) {
          const body: any = event.body;
          if (body && body.code !== 0) {
            // this.msg.error(body.msg);
            if (body.msg === "invalidtoken" || body.msg === "无效token") {
              // this.logout();
            }
            // 继续抛出错误中断后续所有 Pipe、subscribe 操作，因此：
            // this.http.get('/').subscribe() 并不会触发
            return throwError({ msg: body.code });
          } else if (body) {
            // 重新修改 `body` 内容为 `response` 内容，对于绝大多数场景已经无须再关心业务状态码
            return of(
              new HttpResponse(Object.assign(event, { body: body.data }))
            );
            // 或者依然保持完整的格式
            // return of(event);
          } else {
            // this.msg.error('server error！');
          }
        }
        break;
      case 401: // 未登录状态码
        this.goTo("/login");
        break;
      case 403:
      case 404:
      case 500:
        // this.goTo(`/${event.status}`);
        break;
      case 504:
        // this.goTo("/403");
        break;
      default:
        if (event instanceof HttpErrorResponse) {
          console.warn(
            "未可知错误，大部分是由于后端不支持CORS或无效配置引起",
            event
          );
          // this.msg.error(event.message);
        }
        break;
    }
    return of(event);
  }

  intercept(
    req: HttpRequest<any>,
    next: HttpHandler
  ): Observable<
    | HttpSentEvent
    | HttpHeaderResponse
    | HttpProgressEvent
    | HttpResponse<any>
    | HttpUserEvent<any>
  > {
    // 统一加上服务端前缀
    let url = req.url;
    if (!url.startsWith("https://") && !url.startsWith("http://")) {
      url = environment.SERVER_URL + url;
    }

    const newReq = req.clone({
      url: url
    });
    return next.handle(newReq).pipe(
      mergeMap((event: any) => {
        // 允许统一对请求错误处理，这是因为一个请求若是业务上错误的情况下其HTTP请求的状态是200的情况下需要
        if (
          url.indexOf("api/") !== -1 &&
          event instanceof HttpResponse &&
          event.status === 200
        )
          return this.handleData(event);
        // 若一切都正常，则后续操作
        return of(event);
      }),
      catchError((err: HttpErrorResponse) => this.handleData(err))
    );
  }
}
